@@ -5,6 +5,9 @@ flags = argparse.ArgumentParser(description='Take control of your Palette.')
 flags.add_argument('--device', default='16D0:09F8',
                    help='usb device vendor:product id')
 
+flags.add_argument('--device_index', default=0,
+                   help='index of device to control if multiple connected')
+
 flags.add_argument('--debug', default=False, action='store_true',
                    help='include low level protocol messages')
 
